/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#include "oapr/oapr_buildconfig.hpp"
#include "oapr/oapr_pool_mgr.hpp"
#include "oapr/oapr_exception.hpp"

FILE_VERSION("$Revision: 1.6 $",oapr_pool_mgr_cpp);

oapr::pool_mgr *oapr::pool_mgr::m_instance=0;
namespace oapr
{

  /* utility functions */
  bool operator==(pool::pool_ptr a, pool::pool_ptr b){
    return (a->getp()==b->getp() && a->id()==b->id());
  }
  bool operator<(const pool::pool_ptr a, const pool::pool_ptr b){
    return (a->getp()<b->getp() && a->id()<b->id());
  }

  pool_mgr::pool_mgr(const std::string& app)
  {
  }
  pool_mgr::~pool_mgr(){}

  void pool_mgr::initialize(const std::string& app)
  {
    if(!pool_mgr::m_instance){
      pool_mgr::m_instance=new pool_mgr(app);
    }
  }


  pool_mgr* const pool_mgr::instance()
  {
    /**@todo make it thread safe*/
    if(!pool_mgr::m_instance){
      OAPR_THROW_LVL(init_error,"pool manager is not initialized",
		     exception_base::EXC_Emerg);
    }
    return pool_mgr::m_instance;
  }


  void pool_mgr::terminate()
  {
    /**@todo make it thread safe*/
    if(pool_mgr::m_instance){
      pool_mgr*ez=pool_mgr::m_instance;
      ez->m_pools.erase(ez->m_pools.begin(),ez->m_pools.end());
      delete pool_mgr::m_instance;
    }
  }


  pool::pool_ptr pool_mgr::get(const std::string& cat)
  {
    try{
      pool::pool_ptr rv;
      pool_map::iterator fnd=m_pools.find(cat);
      if(fnd == m_pools.end()){
	/* if we don't have a cached one create it*/
	rv.reset(new  pool_t(cat));
	m_pools.insert(std::make_pair(cat,rv));
      }else{
	rv = fnd->second;
      }
      return rv;
    }OAPR_CATCH();
  }

  void pool_mgr::unget(const std::string& cat)
  {
    pool_map::iterator fnd=m_pools.find(cat);
    if(fnd!=m_pools.end()){
      m_pools.erase(fnd);
    }
    /**@todo determine if we should throw if the cat is not found*/
  }
  pool::pool_ptr pool_mgr::unique()
  {
    try{
      pool_ptr rv(new pool_t);
      return rv;
    }OAPR_CATCH();
  }
  
};
