/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#include <iostream>
#include "oapr/oapr_app.hpp"
#include "oapr/oapr_iosockstream.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"

typedef oapr::basic_sockbuf<char,std::char_traits<char> > sockbuf_type;
typedef oapr::basic_iosockstream<char,std::char_traits<char> > sockstream_type;
int main(int argc,char*argv[])
{
  try{
    oapr::oapr_app app("test server stream");
    oapr::pool_mgr::pool_ptr pl=oapr::pool_mgr::instance()->get("socket");
    std::string bindaddr("127.0.0.1"),
      sdata("pong\n"),
      sexit("exit\n"),
      rdata;

    sockstream_type sstrm(pl,bindaddr.c_str(),1234,SOCK_STREAM,
			  sockstream_type::sockbuf_type::server);
    OAPR_CHECK_MSG(sstrm.option(APR_SO_REUSEADDR, 1),
		   "failed to set option");
    sockstream_type *rstrm = sstrm.accept();

    while(*rstrm){
      rdata="";
      std::getline(*rstrm,rdata);
      std::cerr<<"r "<<rdata<<"\n";
      if(!*rstrm)break;
      if(rdata=="ping" && *rstrm){
	*rstrm<<sdata<<std::flush;
	std::cerr<<"w "<<sdata;
      }else if(*rstrm){
	*rstrm<<sexit<<std::flush;
	std::cerr<<"w "<<sexit;
	break;
      }
    }

  }catch(const oapr::oapr_exception &e){
    print_stack(&std::cerr,e);
  }catch(const oapr::exception_base &e){
    print_stack(&std::cerr,e);
  }catch(const std::exception &e){
  }catch(...){
  }
}
