/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

#ifndef OAPR_APPLICATION_HPP
#define OAPR_APPLICATION_HPP

#include <string>
#include "oapr_app.hpp"
#include "oapr_args.hpp"

namespace oapr
{
  /**
   * Application class.
   * Manages the common components of a command-line oapr application.
   */
  class application: public oapr_app
  {
    public:
      application(int argc, char const * const *argv,
		  char const * const *env = NULL,
		  const static_args& argdefs = no_arguments);
      ~application();
      /**
       * Initialize the application. This checks the command-line arguments
       */
      int initialize();

      /* Manual helper methods */
      /**
       * Show command-line usage information.
       */
      void usage();
      /**
       * Check arguments.
       * @retval True if arguments are ok, false if they were bad.
       * @tip If false, show usage and exit.
       */
      bool check_args();
    private:
      int m_argc;
      const char * const *m_argv;
      const char * const *m_env;
      /**\todo m_argc, m_argv, and m_env should all be in oapr_app, as that is where they are
               potentially changed by apr_app_initialize */
      const static_args& m_argdefs; // note: not an array. keep as ref.
  };

};

#endif
